@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * uf_gtk.cc - gtk compatibility layer
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -136,48 +136,6 @@ extern "C" {
         }
     }
 
-// Get the display ICC profile of the monitor associated with the widget.
-// For X display, uses the ICC profile specifications version 0.2 from
-// http://burtonini.com/blog/computers/xicc
-// Based on code from Gimp's modules/cdisplay_lcms.c
-#ifdef GDK_WINDOWING_QUARTZ
-    typedef struct {
-        guchar *data;
-        gsize   len;
-    } ProfileTransfer;
-
-    enum {
-        openReadSpool  = 1, /* start read data process         */
-        openWriteSpool = 2, /* start write data process        */
-        readSpool      = 3, /* read specified number of bytes  */
-        writeSpool     = 4, /* write specified number of bytes */
-        closeSpool     = 5  /* complete data transfer process  */
-    };
-
-    static OSErr _uf_lcms_flatten_profile(SInt32  command,
-                                          SInt32 *size, void *data, void *refCon)
-    {
-        ProfileTransfer *transfer = static_cast<ProfileTransfer*>(refCon);
-
-        switch (command) {
-            case openWriteSpool:
-                g_return_val_if_fail(transfer->data == NULL && transfer->len == 0, -1);
-                break;
-
-            case writeSpool:
-                transfer->data = static_cast<guchar*>(
-                                     g_realloc(transfer->data, transfer->len + *size));
-                memcpy(transfer->data + transfer->len, data, *size);
-                transfer->len += *size;
-                break;
-
-            default:
-                break;
-        }
-        return 0;
-    }
-#endif /* GDK_WINDOWING_QUARTZ */
-
 // On X11 the display profile can be embedded using the 'xicc' command.
     void uf_get_display_profile(GtkWidget *widget,
                                 guint8 **buffer, gint *buffer_size)
@@ -215,13 +173,17 @@ extern "C" {
         if (prof == NULL)
             return;
 
-        ProfileTransfer transfer = { NULL, 0 };
-        Boolean foo;
-        CMFlattenProfile(prof, 0, _uf_lcms_flatten_profile, &transfer, &foo);
+        CFDataRef data;
+        data = CMProfileCopyICCData(NULL, prof);
         CMCloseProfile(prof);
 
-        *buffer = transfer.data;
-        *buffer_size = transfer.len;
+        UInt8 *tmp_buffer = (UInt8 *) g_malloc(CFDataGetLength(data));
+        CFDataGetBytes(data, CFRangeMake(0, CFDataGetLength(data)), tmp_buffer);
+
+        *buffer = (guint8 *) tmp_buffer;
+        *buffer_size = CFDataGetLength(data);
+
+        CFRelease(data);
 
 #elif defined G_OS_WIN32
         (void)widget;
@@ -351,6 +313,8 @@ extern "C" {
             delete data;
             return;
         }
+        if (type != uf_value_changed)
+            return;
         GtkComboBox *combo = GTK_COMBO_BOX(data->gobject[0]);
         UFArray &array = *object;
         if (array.Index() >= 0) {
@@ -576,16 +540,13 @@ extern "C" {
         _UFWidgetData &data = _ufarray_widget_data(array);
         gtk_widget_set_size_request(combo, 50, -1);
         data.gobject[0] = G_OBJECT(combo);
-        char *saveIndex = g_strdup(array.StringValue());
-        int i = 0;
-        while (array.SetIndex(i)) {
-            gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(array.StringValue()));
-            i++;
+        UFGroupList list = array.List();
+        for (UFGroupList::iterator iter = list.begin();
+                iter != list.end(); iter++) {
+            gtk_combo_box_append_text(GTK_COMBO_BOX(combo),
+                                      _((*iter)->StringValue()));
         }
-        array.Set(saveIndex);
-        g_free(saveIndex);
         _ufarray_object_event(object, uf_value_changed);
-        gtk_widget_set_sensitive(combo, i > 0);
         return combo;
     }
 
