@@ -1,8 +1,8 @@
 /*
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
- * ufobject.h - UF-Object definitions.
- * Copyright 2004-2011 by Udi Fuchs
+ * ufobject.h - UFObject definitions.
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -308,6 +308,9 @@ public:
     bool IsEqual(const char *string) const;
 };
 
+/// A list of UFObjects returned by UFGroup or UFArray.
+typedef std::list<UFObject *> UFGroupList;
+
 /**
  * UFGroup is a UFObject that contain a group of UFObject elements. This
  * object is considered the Patent() of these elements.
@@ -336,6 +339,8 @@ public:
     /// \exception UFException is thrown if an element with the given name
     /// does not exist. This can be avoided with the use of the Has() method.
     const UFObject &operator[](UFName name) const;
+    /// Return a list of all UFObjects in the group.
+    const UFGroupList List() const;
     /// Add (append) a UFObject to a UFGroup. If the object belonged to
     /// another group before, it will be detached from the original group.
     /// \exception UFException is thrown if UFGroup already contains
@@ -439,6 +444,8 @@ extern "C" {
     void ufobject_set_user_data(UFObject *object, void *user_data);
     void ufobject_set_changed_event_handle(UFObject *object,
                                            UFEventHandle *handle);
+/// Return TRUE if object is set to its default value.
+    UFBoolean ufobject_is_default(UFObject *object);
 /// Set the current object value to its default value.
     void ufobject_set_default(UFObject *object);
 /// Return the numerical value of the object. Returns NaN if object is not a
