@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_chooser.c - A very simple file chooser for UFRaw.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,8 @@ GtkFileChooser *ufraw_raw_chooser(conf_data *conf,
     gtk_file_filter_set_name(filter, _("Raw images"));
     extList = g_strsplit(raw_ext, ",", 100);
     for (l = extList; *l != NULL; l++)
-        if (strcmp(*l, "jpg") && strcmp(*l, "tif") && strcmp(*l, "ufraw")) {
+        if (strcmp(*l, "jpeg") && strcmp(*l, "jpg") && strcmp(*l, "tif") &&
+                strcmp(*l, "tiff") && strcmp(*l, "ufraw")) {
             snprintf(ext, max_name, "*.%s", *l);
             gtk_file_filter_add_pattern(filter, ext);
             gtk_file_filter_add_pattern(filter, cp = g_ascii_strup(ext, -1));
@@ -95,15 +96,19 @@ GtkFileChooser *ufraw_raw_chooser(conf_data *conf,
     gtk_file_chooser_add_filter(fileChooser, filter);
 
     filter = GTK_FILE_FILTER(gtk_file_filter_new());
-    gtk_file_filter_set_name(filter, _("Raw jpg's"));
+    gtk_file_filter_set_name(filter, _("Raw jpeg's"));
+    gtk_file_filter_add_pattern(filter, "*.jpeg");
     gtk_file_filter_add_pattern(filter, "*.jpg");
+    gtk_file_filter_add_pattern(filter, "*.JPEG");
     gtk_file_filter_add_pattern(filter, "*.JPG");
     gtk_file_chooser_add_filter(fileChooser, filter);
 
     filter = GTK_FILE_FILTER(gtk_file_filter_new());
-    gtk_file_filter_set_name(filter, _("Raw tif's"));
+    gtk_file_filter_set_name(filter, _("Raw tiff's"));
     gtk_file_filter_add_pattern(filter, "*.tif");
+    gtk_file_filter_add_pattern(filter, "*.tiff");
     gtk_file_filter_add_pattern(filter, "*.TIF");
+    gtk_file_filter_add_pattern(filter, "*.TIFF");
     gtk_file_chooser_add_filter(fileChooser, filter);
 
     filter = GTK_FILE_FILTER(gtk_file_filter_new());
@@ -151,6 +156,7 @@ void ufraw_chooser(conf_data *rc, conf_data *conf, conf_data *cmd,
             }
             int status = ufraw_config(uf, rc, conf, cmd);
             if (status == UFRAW_ERROR) {
+                ufraw_close_darkframe(uf->conf);
                 ufraw_close(uf);
             } else {
                 ufraw_preview(uf, rc, FALSE, NULL);
@@ -160,6 +166,9 @@ void ufraw_chooser(conf_data *rc, conf_data *conf, conf_data *cmd,
         }
         g_slist_free(saveList);
     }
+    if (rc->darkframe != NULL)
+        ufraw_close_darkframe(rc);
+
     gtk_widget_destroy(GTK_WIDGET(fileChooser));
     ufraw_message(UFRAW_SET_PARENT, NULL);
 }
