@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_exiv2.cc - read the EXIF data from the RAW file using exiv2.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * Based on a sample program from exiv2 and neftags2jpg.
  *
@@ -73,24 +73,12 @@ extern "C" int ufraw_exif_read_input(ufraw_data *uf)
         /* List of tag names taken from exiv2's printSummary() in actions.cpp */
         Exiv2::ExifData::const_iterator pos;
         /* Read shutter time */
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Photo.ExposureTime")))
-                != exifData.end()) {
+        if ((pos = Exiv2::exposureTime(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->shutterText, max_name, pos, exifData);
             uf->conf->shutter = pos->toFloat();
-        } else if ((pos = exifData.findKey(
-                              Exiv2::ExifKey("Exif.Photo.ShutterSpeedValue")))
-                   != exifData.end()) {
-            uf_strlcpy_to_utf8(uf->conf->shutterText, max_name, pos, exifData);
-            uf->conf->shutter = 1.0 / pos->toFloat();
         }
         /* Read aperture */
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Photo.FNumber")))
-                != exifData.end()) {
-            uf_strlcpy_to_utf8(uf->conf->apertureText, max_name, pos, exifData);
-            uf->conf->aperture = pos->toFloat();
-        } else if ((pos = exifData.findKey(
-                              Exiv2::ExifKey("Exif.Photo.ApertureValue")))
-                   != exifData.end()) {
+        if ((pos = Exiv2::fNumber(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->apertureText, max_name, pos, exifData);
             uf->conf->aperture = pos->toFloat();
         }
@@ -99,12 +87,7 @@ extern "C" int ufraw_exif_read_input(ufraw_data *uf)
             uf_strlcpy_to_utf8(uf->conf->isoText, max_name, pos, exifData);
         }
         /* Read focal length */
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Photo.FocalLength")))
-                != exifData.end()) {
-            uf_strlcpy_to_utf8(uf->conf->focalLenText, max_name, pos, exifData);
-            uf->conf->focal_len = pos->toFloat();
-        } else if ((pos = exifData.findKey(Exiv2::ExifKey(
-                                               "Exif.Canon.FocalLength"))) != exifData.end()) {
+        if ((pos = Exiv2::focalLength(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->focalLenText, max_name, pos, exifData);
             uf->conf->focal_len = pos->toFloat();
         }
@@ -124,17 +107,14 @@ extern "C" int ufraw_exif_read_input(ufraw_data *uf)
             uf_strlcpy_to_utf8(uf->conf->flashText, max_name, pos, exifData);
         }
         /* Read White Balance Setting */
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Photo.WhiteBalance")))
-                != exifData.end()) {
+        if ((pos = Exiv2::whiteBalance(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->whiteBalanceText, max_name, pos, exifData);
         }
 
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Image.Make")))
-                != exifData.end()) {
+        if ((pos = Exiv2::make(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->real_make, max_name, pos, exifData);
         }
-        if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Image.Model")))
-                != exifData.end()) {
+        if ((pos = Exiv2::model(exifData)) != exifData.end()) {
             uf_strlcpy_to_utf8(uf->conf->real_model, max_name, pos, exifData);
         }
 
@@ -232,6 +212,11 @@ static Exiv2::ExifData ufraw_prepare_exifdata(ufraw_data *uf)
             != exifData.end())
         exifData.erase(pos);
 
+    // DCRaw handles TIFF files as raw if DNGVersion is found.
+    if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Image.DNGVersion")))
+            != exifData.end())
+        exifData.erase(pos);
+
     // DNG private data
     if ((pos = exifData.findKey(Exiv2::ExifKey("Exif.Image.DNGPrivateData")))
             != exifData.end())
@@ -304,8 +289,19 @@ extern "C" int ufraw_exif_prepare_output(ufraw_data *uf)
                     != exifData.end()) {
                 exifData.erase(pos);
                 ufraw_message(UFRAW_SET_LOG,
-                              "buflen %d too big, erasing Exif.Photo.MakerNote\n",
+                              "buflen %d too big, erasing Exif.Photo.MakerNote "
+                              "and related decoded metadata\n",
                               size + sizeof(ExifHeader));
+                /* Delete decoded metadata associated with
+                 * Exif.Photo.MakerNote, otherwise erasing it isn't
+                 * effective. */
+                for (pos = exifData.begin(); pos != exifData.end();) {
+                    if (!strcmp(pos->ifdName(), "Makernote"))
+                        pos = exifData.erase(pos);
+                    else
+                        pos++;
+                }
+                blob.clear();
                 Exiv2::ExifParser::encode(blob, Exiv2::bigEndian, exifData);
                 size = blob.size();
             }
@@ -316,6 +312,7 @@ extern "C" int ufraw_exif_prepare_output(ufraw_data *uf)
             ufraw_message(UFRAW_SET_LOG,
                           "buflen %d too big, erasing Thumbnail\n",
                           size + sizeof(ExifHeader));
+            blob.clear();
             Exiv2::ExifParser::encode(blob, Exiv2::bigEndian, exifData);
             size = blob.size();
         }
