@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_lensfun.cc - define lensfun UFObject settings.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +35,13 @@ private:
     static lfDatabase *_LensDB;
 public:
     lfCamera Camera;
-    lfLens Transformation;
+    // 'Interpolation' represents the lens the user choose from the LensDB.
+    // Its transformation values are interpolate according to the choice
+    // of FocalLegth, Aperture and Distance.
     lfLens Interpolation;
+    // 'Transformation' include the transformation the user choose from the
+    // 'Interpolation' lens.
+    lfLens Transformation;
     double FocalLengthValue;
     double ApertureValue;
     double DistanceValue;
@@ -79,12 +84,24 @@ public:
             UFString &CameraModel = (*this)[ufCameraModel];
             CameraModel.Set(fm);
             g_free(fm);
+            // After setting the camera model, we can retry finding the lens.
+            SetLensInterpolation();
         }
     }
     void SetLensModel(const lfLens &lens);
-    void SetInterpolation(const lfLens &lens);
+    // Set the 'Interpolation' lens from the LensModel.
+    void SetLensInterpolation();
+    // Interpolate the TCA, Vignetting and Distortion models.
     void Interpolate();
-    void Init();
+    // Mark settings as manual (not from LensDB).
+    void Manual() {
+        char *lens_model = g_strdup_printf("Generic, Crop factor %.4g",
+                                           Transformation.CropFactor);
+        (*this)[ufLensModel].Set(lens_model);
+        g_free(lens_model);
+        UFObject::Parent()[ufLensfunAuto].Set("no");
+    }
+    void Init(bool reset);
 };
 
 static void parse_maker_model(const char *txt, char *make, size_t sz_make,
@@ -127,22 +144,18 @@ public:
         if (!HasParent())
             return UFObject::Event(type);
         Lensfun &Lensfun = Lensfun::Parent(*this);
-        Lensfun.UFObject::Parent()[ufLensfunAuto].Set("no");
-        char make[200], model[200];
-        parse_maker_model(StringValue(), make, sizeof(make),
-                          model, sizeof(model));
-        const lfLens **lensList = Lensfun.LensDB()->FindLenses(&Lensfun.Camera,
-                                  make, model, LF_SEARCH_LOOSE);
-        if (lensList == NULL) {
-            lfLens emptyLens;
-            Lensfun.SetInterpolation(emptyLens);
-            return UFObject::Event(type);
-        }
-        Lensfun.SetInterpolation(*lensList[0]);
+        Lensfun.SetLensInterpolation();
         Lensfun.Interpolate();
-        lf_free(lensList);
         return UFObject::Event(type);
     }
+    void OriginalValueChangedEvent() {
+        if (!HasParent())
+            return;
+        Lensfun &Lensfun = Lensfun::Parent(*this);
+        // While loading rc/cmd/conf data, do not reset the auto setting
+        if (ufraw_image_get_data(this) != NULL)
+            Lensfun.UFObject::Parent()[ufLensfunAuto].Set("no");
+    }
 };
 
 #define _buffer_size 80
@@ -174,14 +187,26 @@ class FocalLength : public UFArray
 {
 public:
     FocalLength() : UFArray(ufFocalLength) { }
-    void OriginalValueChangedEvent() {
+    void Event(UFEventType type) {
+        if (type == uf_default_changed && Index() == -1) {
+            // Default value is changed during Init. Reset to this default
+            // value if no other value was set.
+            Reset();
+        }
+        if (type != uf_value_changed)
+            return UFObject::Event(type);
+        ufraw_data *uf = ufraw_image_get_data(this);
+        if (uf == NULL)
+            return UFObject::Event(type);
         if (!HasParent())
-            return;
+            return UFObject::Event(type);
         double value;
         if (sscanf(StringValue(), "%lf", &value) != 1)
-            return;
+            return UFObject::Event(type);
         Lensfun::Parent(*this).FocalLengthValue = value;
         Lensfun::Parent(*this).Interpolate();
+        ufraw_invalidate_layer(uf, ufraw_transform_phase);
+        return UFObject::Event(type);
     }
     void CreatePresets() {
         if (!HasParent())
@@ -218,14 +243,25 @@ class Aperture : public UFArray
 {
 public:
     Aperture() : UFArray(ufAperture) { }
-    void OriginalValueChangedEvent() {
+    void Event(UFEventType type) {
+        if (type == uf_default_changed && Index() == -1) {
+            // Default value is changed during Init. Reset to this default
+            // value if no other value was set.
+            Reset();
+        }
+        if (type != uf_value_changed)
+            return UFObject::Event(type);
+        ufraw_data *uf = ufraw_image_get_data(this);
+        if (uf == NULL)
+            return UFObject::Event(type);
         if (!HasParent())
-            return;
+            return UFObject::Event(type);
         double value;
         if (sscanf(StringValue(), "%lf", &value) != 1)
-            return;
+            return UFObject::Event(type);
         Lensfun::Parent(*this).ApertureValue = value;
         Lensfun::Parent(*this).Interpolate();
+        return UFObject::Event(type);
     }
     void CreatePresets() {
         if (!HasParent())
@@ -261,14 +297,25 @@ class Distance : public UFArray
 {
 public:
     Distance() : UFArray(ufDistance) { }
-    void OriginalValueChangedEvent() {
+    void Event(UFEventType type) {
+        if (type == uf_default_changed && Index() == -1) {
+            // Default value is changed during Init. Reset to this default
+            // value if no other value was set.
+            Reset();
+        }
+        if (type != uf_value_changed)
+            return UFObject::Event(type);
+        ufraw_data *uf = ufraw_image_get_data(this);
+        if (uf == NULL)
+            return UFObject::Event(type);
         if (!HasParent())
-            return;
+            return UFObject::Event(type);
         double value;
         if (sscanf(StringValue(), "%lf", &value) != 1)
-            return;
+            return UFObject::Event(type);
         Lensfun::Parent(*this).DistanceValue = value;
         Lensfun::Parent(*this).Interpolate();
+        return UFObject::Event(type);
     }
     void CreatePresets() {
         Clear();
@@ -300,8 +347,11 @@ public:
     void OriginalValueChangedEvent() {
         if (!HasParent())
             return;
-        lfLens emptyLens;
-        Lensfun::Parent(*this)[ufLensModel].Reset();
+        // While loading rc/cmd/conf data, do not reset other settings
+        if (ufraw_image_get_data(this) == NULL)
+            return;
+        Lensfun &Lensfun = Lensfun::Parent(*this);
+        Lensfun.Manual();
     }
 };
 
@@ -361,7 +411,6 @@ public:
         lfLensCalibTCA calib;
         if (!Lensfun.Interpolation.InterpolateTCA(
                     Lensfun.FocalLengthValue, calib)) {
-            SetIndex(0);
             return;
         }
         SetIndex(calib.Model - LF_TCA_MODEL_NONE);
@@ -438,7 +487,6 @@ public:
         if (!Lensfun.Interpolation.InterpolateVignetting(
                     Lensfun.FocalLengthValue, Lensfun.ApertureValue,
                     Lensfun.DistanceValue, calib)) {
-            SetIndex(0);
             return;
         }
         SetIndex(calib.Model - LF_VIGNETTING_MODEL_NONE);
@@ -525,7 +573,6 @@ public:
         lfLensCalibDistortion calib;
         if (!Lensfun.Interpolation.InterpolateDistortion(
                     Lensfun.FocalLengthValue, calib)) {
-            SetIndex(0);
             return;
         }
         SetIndex(calib.Model - LF_DIST_MODEL_NONE);
@@ -556,22 +603,42 @@ public:
             *this << new UFString("Type", typeName);
         }
     }
-    void OriginalValueChangedEvent() {
+    void Event(UFEventType type) {
+        if (type != uf_value_changed)
+            return UFObject::Event(type);
         ufraw_data *uf = ufraw_image_get_data(this);
         if (uf == NULL)
-            return;
+            return UFObject::Event(type);
         Lensfun::Parent(*this).Transformation.Type = lfLensType(Index());
         ufraw_invalidate_layer(uf, ufraw_transform_phase);
+        return UFObject::Event(type);
+    }
+    void OriginalValueChangedEvent() {
+        if (!HasParent())
+            return;
+        // While loading rc/cmd/conf data, do not reset other settings
+        if (ufraw_image_get_data(this) == NULL)
+            return;
+        Lensfun &Lensfun = Lensfun::Parent(*this);
+        Lensfun.Manual();
     }
 };
 
 extern "C" {
     UFName ufTargetLensGeometry = "TargetLensGeometry";
 }
-class TargetLensGeometry : public LensGeometry
+class TargetLensGeometry : public UFArray
 {
 public:
-    TargetLensGeometry() : LensGeometry(ufTargetLensGeometry) { }
+    explicit TargetLensGeometry(UFName name = ufTargetLensGeometry) :
+        UFArray(name, lfLens::GetLensTypeDesc(LF_UNKNOWN, NULL)) {
+        for (lfLensType type = LF_UNKNOWN; ; type = lfLensType(type + 1)) {
+            const char *typeName = lfLens::GetLensTypeDesc(type, NULL);
+            if (typeName == NULL)
+                break; // End of loop.
+            *this << new UFString("Type", typeName);
+        }
+    }
     void OriginalValueChangedEvent() {
         ufraw_data *uf = ufraw_image_get_data(this);
         if (uf == NULL)
@@ -600,9 +667,39 @@ Lensfun::Lensfun() : UFGroup(ufLensfun), FocalLengthValue(0.0),
             ;
 }
 
-void Lensfun::SetInterpolation(const lfLens &lens)
+void Lensfun::SetLensInterpolation()
 {
-    Interpolation = lens;
+    char make[200] = "", model[200] = "";
+    parse_maker_model((*this)[ufLensModel].StringValue(), make, sizeof(make),
+                      model, sizeof(model));
+    double crop_factor = 1.0;
+    int count = sscanf(model, "Crop factor %lf", &crop_factor);
+    if ((strcmp(make, "Generic") == 0 && count == 1) ||
+            (strcmp(make, "") == 0 && strcmp(model, "") == 0)) {
+        lfLens cropLens;
+        cropLens.SetMaker(make);
+        cropLens.SetModel(model);
+        cropLens.CropFactor = crop_factor;
+        cropLens.MinFocal = 1.0;
+        cropLens.MaxFocal = 800.0;
+        UFArray &LensGeometry = (*this)[ufLensGeometry];
+        cropLens.Type = lfLensType(LensGeometry.Index());
+        Interpolation = cropLens;
+    } else {
+        const lfLens **lensList = LensDB()->FindLenses(&Camera,
+                                  make, model, LF_SEARCH_LOOSE);
+        if (lensList == NULL || lensList[0] == NULL) {
+            lfLens emptyLens;
+            Interpolation = emptyLens;
+        } else {
+            Interpolation = *lensList[0];
+        }
+        if (lensList != NULL)
+            lf_free(lensList);
+    }
+    Transformation.CropFactor = Interpolation.CropFactor;
+    UFArray &LensGeometry = (*this)[ufLensGeometry];
+    LensGeometry.SetIndex(Interpolation.Type);
     static_cast<UFRaw::FocalLength &>((*this)[ufFocalLength]).CreatePresets();
     static_cast<UFRaw::Aperture &>((*this)[ufAperture]).CreatePresets();
     static_cast<UFRaw::Distance &>((*this)[ufDistance]).CreatePresets();
@@ -619,7 +716,13 @@ void Lensfun::SetLensModel(const lfLens &lens)
             fm = g_strdup_printf("%s, %s", maker, model);
         else
             fm = g_strdup_printf("%s", model);
-        LensModel.Set(fm);
+        if (LensModel.IsEqual(fm)) {
+            // Even if the lens has not changed, we still want to triger
+            // a change event.
+            LensModel.Event(uf_value_changed);
+        } else {
+            LensModel.Set(fm);
+        }
         g_free(fm);
     }
 }
@@ -633,9 +736,11 @@ void Lensfun::Interpolate()
 
 lfDatabase *Lensfun::_LensDB = NULL;
 
-void Lensfun::Init()
+void Lensfun::Init(bool reset)
 {
     ufraw_data *uf = ufraw_image_get_data(this);
+    if (uf == NULL)
+        return;
     UFGroup &Image = UFObject::Parent();
 
     /* Set lens and camera from EXIF info, if possible */
@@ -658,7 +763,7 @@ void Lensfun::Init()
     UFArray &Distance = (*this)[ufDistance];
     Distance.SetDefault(_StringNumber(buffer, uf->conf->subject_distance));
 
-    if (uf->LoadingID) {
+    if (!reset) {
         (*this)[ufTCA].Event(uf_value_changed);
         (*this)[ufVignetting].Event(uf_value_changed);
         (*this)[ufDistortion].Event(uf_value_changed);
@@ -674,8 +779,14 @@ void Lensfun::Init()
                                     NULL, uf->conf->lensText, LF_SEARCH_LOOSE);
             if (!CameraModel.IsEqual("") && lenses != NULL) {
                 SetLensModel(*lenses[0]);
+                // Changing the lens reset Auto="no". So set it back.
                 LensfunAuto.Set("yes");
                 lf_free(lenses);
+                // When loading a configuration file nothing changes, so we
+                // need to manually trigger the uf_value_changed event.
+                (*this)[ufTCA].Event(uf_value_changed);
+                (*this)[ufVignetting].Event(uf_value_changed);
+                (*this)[ufDistortion].Event(uf_value_changed);
                 return;
             }
         }
@@ -684,8 +795,14 @@ void Lensfun::Init()
                                 NULL, "Standard", LF_SEARCH_LOOSE);
         if (!CameraModel.IsEqual("") && lenses != NULL) {
             SetLensModel(*lenses[0]);
+            // Changing the lens reset Auto="no". So set it back.
             LensfunAuto.Set("yes");
             lf_free(lenses);
+            // When loading a configuration file nothing changes, so we
+            // need to manually trigger the uf_value_changed event.
+            (*this)[ufTCA].Event(uf_value_changed);
+            (*this)[ufVignetting].Event(uf_value_changed);
+            (*this)[ufDistortion].Event(uf_value_changed);
             return;
         }
     }
@@ -697,9 +814,9 @@ void Lensfun::Init()
 
 extern "C" {
 
-    void ufraw_lensfun_init(UFObject *lensfun)
+    void ufraw_lensfun_init(UFObject *lensfun, UFBoolean reset)
     {
-        static_cast<UFRaw::Lensfun *>(lensfun)->Init();
+        static_cast<UFRaw::Lensfun *>(lensfun)->Init(reset);
     }
 
     void ufraw_convert_prepare_transform(ufraw_data *uf,
