@@ -27,6 +27,10 @@
 #ifdef HAVE_LIBZ
 #include <zlib.h>	/* for libpng 1.5.x */
 #endif
+#if PNG_LIBPNG_VER_MAJOR == 1 && (PNG_LIBPNG_VER_MINOR < 5 || \
+    (PNG_LIBPNG_VER_MINOR == 5 && PNG_LIBPNG_VER_RELEASE < 1))
+#define png_const_bytep png_charp
+#endif
 #endif
 
 #ifdef _OPENMP
@@ -198,6 +202,19 @@ int png_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
 }
 #endif /*HAVE_LIBPNG*/
 
+#if defined(HAVE_LIBCFITSIO) && defined(_WIN32)
+/* localtime_r() is not included in the _WIN32 API. */
+static struct tm *localtime_r(const time_t *timep, struct tm *result) {
+    struct tm *p = localtime(timep);
+    memset(result, 0, sizeof(*result));
+    if (p) {
+        *result = *p;
+        p = result;
+    }
+    return p;
+}
+#endif /*HAVE_LIBCFITSIO && _WIN32*/
+
 void ufraw_write_image_data(
     ufraw_data *uf, void * volatile out,
     const UFRectangle *Crop, int bitDepth, int grayscaleMode,
@@ -208,7 +225,8 @@ void ufraw_write_image_data(
     ufraw_image_type *rawImage =
         (ufraw_image_type *)uf->Images[ufraw_first_phase].buffer;
     int byteDepth = (bitDepth + 7) / 8;
-    guint8 pixbuf8[Crop->width * 3 * byteDepth * DEVELOP_BATCH];
+    guint8 *pixbuf8 = g_new(guint8,
+                            Crop->width * 3 * byteDepth * DEVELOP_BATCH);
 
     progress(PROGRESS_SAVE, -Crop->height);
     for (row0 = 0; row0 < Crop->height; row0 += DEVELOP_BATCH) {
@@ -230,6 +248,7 @@ void ufraw_write_image_data(
                        grayscaleMode, bitDepth) != UFRAW_SUCCESS)
             break;
     }
+    g_free(pixbuf8);
 }
 
 int ufraw_write_image(ufraw_data *uf)
@@ -541,7 +560,7 @@ int ufraw_write_image(ufraw_data *uf)
                     png_set_iCCP(png, info,
                                  uf->developer->profileFile[out_profile],
                                  PNG_COMPRESSION_TYPE_BASE,
-                                 buf, len);
+                                 (png_const_bytep) buf, len);
                     g_free(buf);
                 } else {
                     ufraw_set_warning(uf,
@@ -560,7 +579,7 @@ int ufraw_write_image(ufraw_data *uf)
                                  uf->conf->profile[out_profile]
                                  [uf->conf->profileIndex[out_profile]].name,
                                  PNG_COMPRESSION_TYPE_BASE,
-                                 buf, len);
+                                 (png_const_bytep) buf, len);
                 } else {
                     ufraw_set_warning(uf,
                                       _("Failed to embed output profile '%s' in '%s'."),
@@ -825,8 +844,8 @@ static void PNGwriteRawProfile(png_struct *ping,
     *dp++ = '\n';
     *dp = '\0';
 
-#if ((PNG_LIBPNG_VER_MAJOR > 1) || ((PNG_LIBPNG_VER_MAJOR == 1) && \
-(PNG_LIBPNG_VER_MINOR > 2))) && (defined(INT_MAX) && (INT_MAX > 0x7ffffffeL))
+#if (PNG_LIBPNG_VER_MAJOR > 1 || (PNG_LIBPNG_VER_MAJOR == 1 && \
+    PNG_LIBPNG_VER_MINOR > 2)) && (defined(INT_MAX) && INT_MAX > 0x7ffffffeL)
     g_snprintf(dp, allocated_length - strlen(text[0].text), "%8u ", length);
 #else
     g_snprintf(dp, allocated_length - strlen(text[0].text), "%8lu ", length);
